%{
###################### README ######################

Axis directions:
x: medio-lateral axis => flexion axis
y: anterorposterior axis (+: anterior) => adduction/abduction / patellar spin
z: proximodistal axis (+: proximal) => axial rotation / patellar tilt

Input Data:
Matlab Data Files (.mat) with one line per Frame and 4 Columns:

Type 1-6: Cardan Angles.
Column 1                        Column 2    Column 3    Column 4
Time/Tibiofemoral Flexion      X Rotation  Y-Rotation  Z-Rotation
=> Columns independent of rotation sequence

Type 7-12: Euler Angles.
Column 1                        Column 2      Column 3      Column 4
Time/Tibiofemoral Flexion    1. Rotation   2. Rotation   3. Rotation
=> Columns in order of sequence. Axis of 1. and 3. Rotation is the same

Options to choose:
1. Input-Data-Type according to your Input-File
2. one or several conventions for Output
3. x-Axis Data-Type for the charts according to your input data
###################################################
%}

%% Main
clear all
close all
global OutputProjData InputTypeChoosen OutputTypesChoosen GUIDone idxOutProjected xAxisDataType

[file,path] = uigetfile({'*.mat'},'Please select the .mat file with the input data.');

Imported = load([path file]); % Intrinische XYZ-Sequenz in Spalten 5-7 (Cardan Angles XYZ)

LoadedFileNames = fieldnames(Imported);

ImpVar = Imported.(LoadedFileNames{1});

ChartXAxisData = ImpVar(:,1);
FullData = ImpVar(:,2:4);

clear Imported LoadedFileNames file path ImpVar

GUIDone = 0;
idxOutProjected = 7;
    
OptionsGUI; % Start User Input Window for Input/Output-types

while ishandle(1) == true % wait until options-window is closed
    pause(0.5);   
end

if GUIDone ~= 1 % 2 If Cancel-Button was pressed, 0 if figure closed manually, 1 if OK was pressed
    return
end

if InputTypeChoosen == 13
    disp('Projektionen :)');
end

[NumSteps,~] = size(FullData); 
[~,NumOut] = size(OutputTypesChoosen);

% Prepare variables for results
legends = {};
for j = 1:NumOut
    switch OutputTypesChoosen(1,j)
        case 1
            ResultsXYZ = [];
            legends{end+1} = 'Cardan: XYZ';
        case 2
            ResultsXZY = [];
            legends{end+1} = 'Cardan: XZY';
        case 3
            ResultsYXZ = [];
            legends{end+1} = 'Cardan: YXZ';
        case 4
            ResultsYZX = [];
            legends{end+1} = 'Cardan: YZX';
        case 5
            ResultsZXY = [];
            legends{end+1} = 'Cardan: ZXY';
        case 6
            ResultsZYX = [];
            legends{end+1} = 'Cardan: ZYX';
        case 7
            ResultsProj = [];
            legendsFlex = legends;
            legendsRot = legends;
            legendsTilt = legends;
            for p = 1:12
                if OutputProjData(p,1) == 1
                    if p == 1
                        legendsFlex{end+1} = 'Y on Patella';
                    elseif p == 2
                        legendsFlex{end+1} = 'Z on Patella';
                    elseif p == 3
                        legendsFlex{end+1} = 'Y on Femur';
                    elseif p == 4
                        legendsFlex{end+1} = 'Z on Femur';
                    elseif p == 5
                        legendsRot{end+1} = 'X on Patella';
                    elseif p == 6
                        legendsRot{end+1} = 'Z on Patella';
                    elseif p == 7
                        legendsRot{end+1} = 'X on Femur';
                    elseif p == 8
                        legendsRot{end+1} = 'Z on Femur';
                    elseif p == 9
                        legendsTilt{end+1} = 'X on Patella';
                    elseif p == 10
                        legendsTilt{end+1} = 'Y on Patella';
                    elseif p == 11
                        legendsTilt{end+1} = 'X on Femur';
                    elseif p == 12
                        legendsTilt{end+1} = 'Y on Femur';
                    end                    
                end         
            end
        otherwise
            disp('No output type choosen.')
    end
end

if ismember(idxOutProjected,OutputTypesChoosen) == false
    legendsFlex = legends;
    legendsRot = legends;
    legendsTilt = legends;
end

% Calculate results for every timestep
for i = 1:NumSteps
    Data = FullData(i,:);

    R = FunGetR(Data,InputTypeChoosen);

    for j = 1:NumOut
        switch OutputTypesChoosen(1,j)
            case 1
                CardAn = FunCardAn(R,1,2,3);
                ResultsXYZ = [ResultsXYZ; CardAn];
            case 2
                CardAn = FunCardAn(R,1,3,2);
                ResultsXZY = [ResultsXZY; CardAn];
            case 3
                CardAn = FunCardAn(R,2,1,3);
                ResultsYXZ = [ResultsYXZ; CardAn];
            case 4
                CardAn = FunCardAn(R,2,3,1);
                ResultsYZX = [ResultsYZX; CardAn];
            case 5
                CardAn = FunCardAn(R,3,1,2);
                ResultsZXY = [ResultsZXY; CardAn];
            case 6
                CardAn = FunCardAn(R,3,2,1);
                ResultsZYX = [ResultsZYX; CardAn];
            case 7
                for p = 1:12
                    if OutputProjData(p,1) == 1
                        ProjAn = FunProjAn(R,p);
                        ResultsProj(i,p) = ProjAn;
                    else
                        ResultsProj(i,p) = 0;
                    end
                end
        end
    end    
end

%% Plot

figure('Name','Flexion','units','normalized','outerposition',[0.005 0.1 0.33 0.5]);
hold on
for j = 1:NumOut
    switch OutputTypesChoosen(1,j)
        case 1
            plot(ChartXAxisData,ResultsXYZ(:,1));
        case 2
            plot(ChartXAxisData,ResultsXZY(:,1));
        case 3
            plot(ChartXAxisData,ResultsYXZ(:,1));
        case 4
            plot(ChartXAxisData,ResultsYZX(:,1));
        case 5
            plot(ChartXAxisData,ResultsZXY(:,1));
        case 6
            plot(ChartXAxisData,ResultsZYX(:,1));
        case 7
            for p = 1:4
                if OutputProjData(p,1) == 1
                    plot(ChartXAxisData,ResultsProj(:,p));
                end
            end
    end
end
legend(legendsFlex,'Location','southoutside','NumColumns',2);
title('Patellar flexion');
if xAxisDataType == 1
    xlabel('Time');
elseif xAxisDataType == 2
    xlabel('Tibiofemoral flexion');
end
ylabel('Patellar Flexion');
%set(gcf, 'Position',  [100, 100, 750, 500]);
hold off

figure('Name','Spin','units','normalized','outerposition',[0.335 0.1 0.33 0.5]);
hold on
for j = 1:NumOut
    switch OutputTypesChoosen(1,j)
        case 1
            plot(ChartXAxisData,ResultsXYZ(:,2));
        case 2
            plot(ChartXAxisData,ResultsXZY(:,2));
        case 3
            plot(ChartXAxisData,ResultsYXZ(:,2));
        case 4
            plot(ChartXAxisData,ResultsYZX(:,2));
        case 5
            plot(ChartXAxisData,ResultsZXY(:,2));
        case 6
            plot(ChartXAxisData,ResultsZYX(:,2));
        case 7
            for p = 5:8
                if OutputProjData(p,1) == 1
                    plot(ChartXAxisData,ResultsProj(:,p));
                end
            end
    end
end
legend(legendsRot,'Location','southoutside','NumColumns',2);
title('Patellar Spin');
if xAxisDataType == 1
    xlabel('Time');
elseif xAxisDataType == 2
    xlabel('Tibiofemoral flexion');
end
ylabel('Patellar Spin');
%set(gcf, 'Position',  [1000, 100, 750, 500]);
hold off

figure('Name','Tilt','units','normalized','outerposition',[0.665 0.1 0.33 0.5]);
hold on
for j = 1:NumOut
    switch OutputTypesChoosen(1,j)
        case 1
            plot(ChartXAxisData,ResultsXYZ(:,3));
        case 2
            plot(ChartXAxisData,ResultsXZY(:,3));
        case 3
            plot(ChartXAxisData,ResultsYXZ(:,3));
        case 4
            plot(ChartXAxisData,ResultsYZX(:,3));
        case 5
            plot(ChartXAxisData,ResultsZXY(:,3));
        case 6
            plot(ChartXAxisData,ResultsZYX(:,3));
        case 7
            for p = 9:12
                if OutputProjData(p,1) == 1
                    plot(ChartXAxisData,ResultsProj(:,p));
                end
            end
    end
end
legend(legendsTilt,'Location','southoutside','NumColumns',2);
title('Patellar Tilt');
if xAxisDataType == 1
    xlabel('Time');
elseif xAxisDataType == 2
    xlabel('Tibiofemoral flexion');
end
ylabel('Patellar Tilt');
%set(gcf, 'Position',  [2000, 100, 750, 500]);
hold off

clear CardAn Data GUIDone i idxOutProjected InpType InputProjData InputTypeChoosen InputTypes j legends...
    NumOut NumPairs NumSteps OutputProjData OutputTypes OutputTypesChoosen OutTypes p PairMat...
    PairMats path PatKin ProjAn R legendsFlex legendsRot legendsTilt xAxisDataType FullData


%% Functions

function Rx = FunRx(alpha)
    Rx = [1, 0, 0;
          0, cosd(alpha), -sind(alpha);
          0, sind(alpha), cosd(alpha)];
end

function Ry = FunRy(alpha)
    Ry = [cosd(alpha), 0, sind(alpha);
          0, 1, 0;
          -sind(alpha), 0, cosd(alpha)]; 
end

function Rz = FunRz(alpha)
    Rz = [cosd(alpha), -sind(alpha), 0;
          sind(alpha), cosd(alpha), 0;
          0, 0, 1];
end

function R = FunGetR(Data,InpType)
    alpha = Data(1,1);
    beta = Data(1,2);
    gamma = Data(1,3);
    
    switch InpType
        case 1
            R = FunRx(alpha)*FunRy(beta)*FunRz(gamma);
        case 2
            R = FunRx(alpha)*FunRz(gamma)*FunRy(beta);
        case 3
            R = FunRy(beta)*FunRx(alpha)*FunRz(gamma);
        case 4
            R = FunRy(beta)*FunRz(gamma)*FunRx(alpha);
        case 5
            R = FunRz(gamma)*FunRx(alpha)*FunRy(beta);
        case 6
            R = FunRz(gamma)*FunRy(beta)*FunRx(alpha);
        case 7
            R = FunRx(alpha)*FunRy(beta)*FunRx(gamma);
        case 8
            R = FunRy(alpha)*FunRx(beta)*FunRy(gamma);
        case 9
            R = FunRx(alpha)*FunRz(beta)*FunRx(gamma);
        case 10
            R = FunRz(alpha)*FunRx(beta)*FunRz(gamma);
        case 11
            R = FunRy(alpha)*FunRz(beta)*FunRy(gamma);
        case 12
            R = FunRz(alpha)*FunRy(beta)*FunRz(gamma);
        otherwise
            disp('Invalid input type')
            R = zeros(3,3);
    end
end

function CardAn = FunCardAn(R,i,j,k)
    % Calculates cardan angles to the sequence i,j,k
    tupel = [i,j,k];
    CardAn = zeros(1,3);
    % Check, if all three indices are given in (i,j,k) if not: return (0,0,0) and a error message in the command window.
    if sum(ismember(tupel,1))*sum(ismember(tupel,2))*sum(ismember(tupel,3)) ~= 1
        disp('Error in FunCardAn: i,j,k are not unique and in (1,2,3).')
        return;
    end
    
    % Calculate sign of (i,j,k)
    % number of pairs with first Index in tuple < second is odd if and only if sgn(i,j,k) = 1
    count = 0;
    for ii = 1:2
        for jj = ii+1:3
            if tupel(ii) < tupel(jj)
                count = count + 1;
            end
        end
    end
    if mod(count,2) == 1
        sgn = 1;
    else
        sgn = -1;
    end
    
    % Calculate the angles (in degree)
    CardAn(1,i) = atan2d(R(j,k)*-sgn,R(k,k));
    CardAn(1,j) = asind(R(i,k)*sgn);
    CardAn(1,k) = atan2d(R(i,j)*-sgn,R(i,i));
end

function ProjAn = FunProjAn(R,p)
    % Calculates the projected angles of the Rotation R
    % Variables: 
    % p gives the number of the case, defined by "Axes" and "RefSyst".
    % The choosen vectors are projected on the planes perpendicular to the respective rotation axis.
    
    % Get case:
    i = floor((p-1)/4)+1; % i = 1: calculate flexion, i = 2: rotation, i = 3: tilt , => i: axis around which we rotate.
    Axes = [2 3 2 3 1 3 1 3 1 2 1 2]; % Axis to project
    RefSyst = [1 1 2 2 1 1 2 2 1 1 2 2]; % planes on which we project (1: patellasyst, 2: femursyst)
    
    % Calculate angles
    ProjAn = 0;
    
    if RefSyst(p) == 2 % Projection on femoral planes => use R
        R4Use = R;
    elseif RefSyst(p) == 1 % Projection on patellar planes => need femoral standard-vectors in patellar coordinates
                           % => Columns of inverted R (which is transposed R)
        R4Use = R';
    end

    ProjVec = R4Use(:,Axes(p)); % Vector that needs to be projected is i-th column of rotation matrix
    ProjVec(i,1) = 0; % Project on i-th plane
    ProjVec = ProjVec./norm(ProjVec); % normalize projected Vector
    ProjAn = acosd(ProjVec(Axes(p),1)); % Calculates the smallest angle between projected vector and standard vector

    % find the right sign for the angle
    if ProjVec(6-i-Axes(p),1) < 0 % if remaining coordinate of projected vector is negative => change sign
        ProjAn = -1 * ProjAn;
    end

    CoordOrder = [Axes(p), 0, i]; % first axis: Projected vector, third: rotation axis
    CoordOrder(1,2) = 6-sum(CoordOrder); % second vector: the other axis
    % Check sign of this order (positive: right handed, negative: left handed)
    % If left handed: invert sign of the angle after coosing the quadrant
    count = 0;
    for ii = 1:2
        for jj = ii+1:3
            if CoordOrder(ii) < CoordOrder(jj)
                count = count + 1;
            end
        end
    end
    if mod(count,2) == 1
        sgn = 1;
    else
        sgn = -1;
    end

    ProjAn = sgn * ProjAn; % Choose final sign of the angle.
end

function GUI = OptionsGUI
    % Graphical user interface for Options (input & output types)
    global idxOutProjected GUIDone InputTypeChoosen OutputTypesChoosen OutputProjData xAxisDataType
    
    % Initialize
    InputTypeChoosen = 1;
    OutputTypesChoosen = [1];
    OutputProjData = zeros(12,1);
    
    fig = figure('Name','TransformOptions','NumberTitle','off');
    fig.Position = [200, 200, 1000, 350];
    % Naming for Checkboxes: Angle - projected axis - System on which we project
   
    % Texts
    Title = uicontrol(fig,'Style','text','String','Options','HorizontalAlignment','left',...
        'Position',[30 300 200 30],'Fontsize',15);
    SubtitleInputData = uicontrol(fig,'Style','text','String','Please choose the definition of your input data.',...
        'HorizontalAlignment','left','Position',[30 275 300 20],'Fontsize',10);
    SubtitleOutputData = uicontrol(fig,'Style','text','String','Please choose at least one output format.',...
        'HorizontalAlignment','left','Position',[530 275 300 20],'Fontsize',10);
    
    SubTitleOutputProjProp = uicontrol(fig,'Style','text','String','Properties of projected angles (output)','HorizontalAlignment','left',...
        'Position',[530 115 300 20],'Fontsize',10,'Visible','off');   
    
    % Option Fields
    InputTypes = {'Cardan: Flex - Rotation - Tilt ( = Three-Cylinder open-chain representation)',...
          'Cardan: Flex - Tilt - Rotation','Cardan: Rotation - Flexion - Tilt',...                   
            'Cardan: Rotation - Tilt - Flexion','Cardan: Tilt - Flexion - Rotation',...
            'Cardan: Tilt - Rotation - Flexion','Euler: XYX','Euler: YXY','Euler: XZX',...
            'Euler: ZXZ','Euler: YZY','Euler: ZYZ'};
    ChooseInputType = uicontrol(fig,'Style','listbox','String',InputTypes,'Position',[30 60 450 205],...
        'Fontsize',10,'Max',20,'Min',1,...
        'Callback', @(ChooseInputType,event) ChooseInputTypeChanged(ChooseInputType));  
    
    OutputTypes = {'Cardan: Flex - Rotation - Tilt ( = Three-Cylinder open-chain representation)',...
        'Cardan: Flex - Tilt - Rotation','Cardan: Rotation - Flexion - Tilt !NOT RECOMMENDED!',...                   
        'Cardan: Rotation - Tilt - Flexion','Cardan: Tilt - Flexion - Rotation !NOT RECOMMENDED!',...
        'Cardan: Tilt - Rotation - Flexion','Projected Angles'};
    ChooseOutputTypes = uicontrol(fig,'Style','listbox','String',OutputTypes,'Position',[530 145 450 120],...
        'Fontsize',10,'Max',20,'Min',1,...
        'Callback', @(ChooseOutputTypes,event) ChooseOutputTypesChanged(ChooseOutputTypes)); 
    
   OutputFlexGroup = uibuttongroup('Units','pixels','Position',[530 15 100 100],...
              'Title','Flexion','Visible','off');
    
        OutFlex1 = uicontrol(OutputFlexGroup,'Style','checkbox','String','Y on patella','Position',[10 65 85 15],...
            'Callback',@(OutFlex1,event) OutFlex1Change(OutFlex1));
        OutFlex2 = uicontrol(OutputFlexGroup,'Style','checkbox','String','Z on patella','Position',[10 45 85 15],...
            'Callback',@(OutFlex2,event) OutFlex2Change(OutFlex2));
        OutFlex3 = uicontrol(OutputFlexGroup,'Style','checkbox','String','Y on femur','Position',[10 25 85 15],...
            'Callback',@(OutFlex3,event) OutFlex3Change(OutFlex3));
        OutFlex4 = uicontrol(OutputFlexGroup,'Style','checkbox','String','Z on femur','Position',[10 5 85 15],...
            'Callback',@(OutFlex4,event) OutFlex4Change(OutFlex4));

    OutputRotGroup = uibuttongroup('Units','pixels','Position',[640 15 100 100],...
              'Title','Rotation','Visible','off');

        OutRot1 = uicontrol(OutputRotGroup,'Style','checkbox','String','X on patella','Position',[10 65 85 15],...
            'Callback',@(OutRot1,event) OutRot1Change(OutRot1));
        OutRot2 = uicontrol(OutputRotGroup,'Style','checkbox','String','Z on patella','Position',[10 45 85 15],...
            'Callback',@(OutRot2,event) OutRot2Change(OutRot2));
        OutRot3 = uicontrol(OutputRotGroup,'Style','checkbox','String','X on femur','Position',[10 25 85 15],...
            'Callback',@(OutRot3,event) OutRot3Change(OutRot3));
        OutRot4 = uicontrol(OutputRotGroup,'Style','checkbox','String','Z on femur','Position',[10 5 85 15],...
            'Callback',@(OutRot4,event) OutRot4Change(OutRot4));

    OutputTiltGroup = uibuttongroup('Units','pixels','Position',[750 15 100 100],...
              'Title','Tilt','Visible','off');

        OutTilt1 = uicontrol(OutputTiltGroup,'Style','checkbox','String','X on patella','Position',[10 65 85 15],...
            'Callback',@(OutTilt1,event) OutTilt1Change(OutTilt1));
        OutTilt2 = uicontrol(OutputTiltGroup,'Style','checkbox','String','Y on patella','Position',[10 45 85 15],...
            'Callback',@(OutTilt2,event) OutTilt2Change(OutTilt2));
        OutTilt3 = uicontrol(OutputTiltGroup,'Style','checkbox','String','X on femur','Position',[10 25 85 15],...
            'Callback',@(OutTilt3,event) OutTilt3Change(OutTilt3));
        OutTilt4 = uicontrol(OutputTiltGroup,'Style','checkbox','String','Y on femur','Position',[10 5 85 15],...
            'Callback',@(OutTilt4,event) OutTilt4Change(OutTilt4));
      
    % Ok-Button
    OKbtn = uicontrol(fig,'Style','pushbutton','String', 'OK','Position',[30, 15, 100, 30],'Fontsize',10,...
                   'Callback', @(OKbtn,event) OKButtonPushed(OKbtn));
    
    % Cancel-Button
    Cancelbtn = uicontrol(fig,'Style','pushbutton','String', 'Cancel','Position',[160, 15, 100, 30],'Fontsize',10,...
                   'Callback', @(Cancelbtn,event) CancelButtonPushed(Cancelbtn));
       
     
    % Create Radio-Button-Group for choice of type of x-Axis (Flexion angle or Time
    XAxisGroup = uibuttongroup('Units','pixels','Position',[300 15 160 40],...
                'Title','Datatype for x-axis');
              
    % Create the radio buttons in the button group.
    TimeUI = uicontrol(XAxisGroup,'Style','radiobutton','String','Time','Position',[10 5 65 15]);

    FlexionUI = uicontrol(XAxisGroup,'Style','radiobutton','String','Flexion','Position',[80 5 65 15]);

        
               
               
    % Embedded Functions
        
    function ChooseInputTypeChanged(ChooseInputType)
        InputTypeChoosen = get(ChooseInputType,'Value');
    end

   function ChooseOutputTypesChanged(ChooseOutputTypes) % if projected angles are chosen: Show options
        OutputTypesChoosen = get(ChooseOutputTypes,'Value');
        if ismember(idxOutProjected,OutputTypesChoosen) == true
            SubTitleOutputProjProp.Visible = 'on';
            OutputFlexGroup.Visible = 'on' ;
            OutputRotGroup.Visible = 'on';
            OutputTiltGroup.Visible = 'on';
        else
            SubTitleOutputProjProp.Visible = 'off';
            OutputFlexGroup.Visible = 'off' ;
            OutputRotGroup.Visible = 'off';
            OutputTiltGroup.Visible = 'off';
            OutputProjData = zeros(12,1);
        end
   end

    % Functions: Checkboxes projected Angles for Output
    function OutFlex1Change(OutFlex1)
        OutputProjData(1,1) = get(OutFlex1, 'Value');
    end

    function OutFlex2Change(OutFlex2)
        OutputProjData(2,1) = get(OutFlex2, 'Value');
    end

    function OutFlex3Change(OutFlex3)
        OutputProjData(3,1) = get(OutFlex3, 'Value');
    end

    function OutFlex4Change(OutFlex4)
        OutputProjData(4,1) = get(OutFlex4, 'Value');
    end

    
    function OutRot1Change(OutRot1)
        OutputProjData(5,1) = get(OutRot1, 'Value');
    end

    function OutRot2Change(OutRot2)
        OutputProjData(6,1) = get(OutRot2, 'Value');
    end

    function OutRot3Change(OutRot3)
        OutputProjData(7,1) = get(OutRot3, 'Value');
    end

    function OutRot4Change(OutRot4)
        OutputProjData(8,1) = get(OutRot4, 'Value');
    end


    function OutTilt1Change(OutTilt1)
        OutputProjData(9,1) = get(OutTilt1, 'Value');
    end

    function OutTilt2Change(OutTilt2)
        OutputProjData(10,1) = get(OutTilt2, 'Value');
    end

    function OutTilt3Change(OutTilt3)
        OutputProjData(11,1) = get(OutTilt3, 'Value');
    end

    function OutTilt4Change(OutTilt4)
        OutputProjData(12,1) = get(OutTilt4, 'Value');
    end

    function OKButtonPushed(OKbtn)
        % Get Datatype for x-Axis
        if TimeUI.Value == true
             xAxisDataType = 1;
        elseif FlexionUI.Value == true
            xAxisDataType = 2;
        end   
        GUIDone = 1;
        closereq 
    end

    function CancelButtonPushed(Cancelbtn)   
        GUIDone = 2;
        closereq 
    end
end